"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var test_1 = require("@platform/test");
var chai_1 = require("chai");
var rxjs_1 = require("rxjs");
var _1 = require(".");
var common_1 = require("../common");
var TEST_DIR = './tmp/test';
var cleanUp = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
    return [2, common_1.fs.remove(TEST_DIR)];
}); }); };
describe('Template', function () {
    beforeEach(cleanUp);
    afterEach(cleanUp);
    describe('create', function () {
        it('it has no sources by default', function () {
            var tmpl = _1.Template.create();
            chai_1.expect(tmpl.sources).to.eql([]);
        });
        it('takes source within `create` method (single)', function () {
            var source = { dir: '.' };
            var tmpl = _1.Template.create(source);
            chai_1.expect(tmpl.sources.length).to.eql(1);
            chai_1.expect(tmpl.sources[0]).to.eql(source);
        });
        it('takes sources within `create` method (array)', function () {
            var sources = [
                { dir: './tmpl-1' },
                { dir: './tmpl-2/foo', pattern: '*.ts' },
            ];
            var tmpl = _1.Template.create(sources);
            chai_1.expect(tmpl.sources).to.eql(sources);
        });
        it('takes sources within `create` method as simple strings (array)', function () {
            var tmpl = _1.Template.create(['./tmpl-1', './tmpl-2/foo']);
            chai_1.expect(tmpl.sources).to.eql([
                { dir: './tmpl-1' },
                { dir: './tmpl-2/foo' },
            ]);
        });
    });
    describe('add (source template)', function () {
        it('adds as a new instance', function () {
            var tmpl1 = _1.Template.create({ dir: './tmpl-1' });
            var tmpl2 = tmpl1.add({ dir: './tmpl-2' });
            chai_1.expect(tmpl1).to.not.equal(tmpl2);
            chai_1.expect(tmpl1.sources).to.eql([{ dir: './tmpl-1' }]);
            chai_1.expect(tmpl2.sources).to.eql([{ dir: './tmpl-1' }, { dir: './tmpl-2' }]);
        });
        it('add from string', function () {
            var tmpl = _1.Template.create().add('./tmpl-1');
            chai_1.expect(tmpl.sources).to.eql([{ dir: './tmpl-1' }]);
        });
        it('add from string array', function () {
            var tmpl = _1.Template.create().add(['./tmpl-1', './tmpl-2']);
            chai_1.expect(tmpl.sources).to.eql([{ dir: './tmpl-1' }, { dir: './tmpl-2' }]);
        });
        it('chaining', function () {
            var tmpl = _1.Template.create()
                .add({ dir: './tmpl-1' })
                .add({ dir: './tmpl-2' });
            chai_1.expect(tmpl.sources).to.eql([{ dir: './tmpl-1' }, { dir: './tmpl-2' }]);
        });
        it('prevents adding the same template more than once', function () {
            var tmpl = _1.Template.create()
                .add({ dir: './tmpl-1', pattern: '*.ts' })
                .add({ dir: './tmpl-1', pattern: '*.ts' });
            chai_1.expect(tmpl.sources).to.eql([{ dir: './tmpl-1', pattern: '*.ts' }]);
        });
        it('merges in another [tmpl]', function () {
            var tmpl1 = _1.Template.create({ dir: './tmpl-1' });
            var tmpl2 = _1.Template.create([{ dir: './tmpl-2' }, { dir: './tmpl-3' }]);
            var res = tmpl1.add(tmpl2);
            chai_1.expect(res.sources).to.eql([
                { dir: './tmpl-1' },
                { dir: './tmpl-2' },
                { dir: './tmpl-3' },
            ]);
        });
        it('merges multiple [tmpl] items', function () {
            var tmpl1 = _1.Template.create({ dir: './tmpl-1' });
            var tmpl2 = _1.Template.create(['./tmpl-2', './tmpl-4']);
            var tmpl3 = _1.Template.create().add('./tmpl-3');
            var tmpl4 = _1.Template.create().add('./tmpl-4');
            var tmpl5 = _1.Template.create().add({
                dir: './tmpl-4',
                targetDir: '/foo',
            });
            var res = tmpl1.add([tmpl2, tmpl3, tmpl3, tmpl1, tmpl4, tmpl5, tmpl5]);
            chai_1.expect(res.sources).to.eql([
                { dir: './tmpl-1' },
                { dir: './tmpl-2' },
                { dir: './tmpl-4' },
                { dir: './tmpl-3' },
                { dir: './tmpl-4', targetDir: '/foo' },
            ]);
        });
    });
    describe('files', function () {
        it('has no files (dir does not exist)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({
                            dir: './NO_EXIST',
                            pattern: '**',
                        });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        chai_1.expect(files.length).to.eql(0);
                        return [2];
                }
            });
        }); });
        it('has no files (file does not exist)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({
                            dir: './example/tmpl-1',
                            pattern: 'NO_EXIST.md',
                        });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        chai_1.expect(files.length).to.eql(0);
                        return [2];
                }
            });
        }); });
        it('has no files (empty dir)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({
                            dir: './example/empty',
                            pattern: '**',
                        });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        chai_1.expect(files.length).to.eql(0);
                        return [2];
                }
            });
        }); });
        it('has single file', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var source, tmpl, files, file;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        source = {
                            dir: './example/tmpl-1',
                            pattern: 'README.md',
                        };
                        tmpl = _1.Template.create(source);
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        file = files[0];
                        chai_1.expect(files.length).to.eql(1);
                        chai_1.expect(file.base).to.eql(common_1.fsPath.resolve(source.dir));
                        chai_1.expect(file.source).to.eql('/README.md');
                        return [2];
                }
            });
        }); });
        it('has multiple files ("**" glob pattern by default)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, includes, files;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({ dir: './example/tmpl-1' });
                        includes = function (paths) {
                            chai_1.expect(paths).to.include('/.babelrc');
                            chai_1.expect(paths).to.include('/.gitignore');
                            chai_1.expect(paths).to.include('/README.md');
                            chai_1.expect(paths).to.include('/images/face.svg');
                            chai_1.expect(paths).to.include('/src/index.ts');
                        };
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        includes(files.map(function (f) { return f.source; }));
                        includes(files.map(function (f) { return f.target; }));
                        return [2];
                }
            });
        }); });
        it('filter pattern (.ts file only)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files, paths;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({
                            dir: './example/tmpl-1',
                            pattern: '**/*.ts',
                        });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        paths = files.map(function (f) { return f.source; });
                        chai_1.expect(paths).to.eql(['/index.ts', '/src/index.ts']);
                        return [2];
                }
            });
        }); });
        it('converts a `dir` that is a file-path to a {dir, pattern}', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({ dir: './example/tmpl-1/README.md' });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        chai_1.expect(files.length).to.eql(1);
                        chai_1.expect(files[0].source).to.eql('/README.md');
                        return [2];
                }
            });
        }); });
        it('throws if a full file-path was given to `dir` AND a pattern was specified', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl;
            return tslib_1.__generator(this, function (_a) {
                tmpl = _1.Template.create().add({
                    dir: './example/tmpl-1/README.md',
                    pattern: '*.ts',
                });
                test_1.expectError(function () { return tmpl.files(); });
                return [2];
            });
        }); });
        it('caches results', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files1, files2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({ dir: './example/tmpl-1' });
                        return [4, tmpl.files()];
                    case 1:
                        files1 = _a.sent();
                        return [4, tmpl.files()];
                    case 2:
                        files2 = _a.sent();
                        chai_1.expect(files1).to.equal(files2);
                        return [2];
                }
            });
        }); });
        it('forces new results (override cache)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files1, files2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create({ dir: './example/tmpl-1' });
                        return [4, tmpl.files()];
                    case 1:
                        files1 = _a.sent();
                        return [4, tmpl.files({ cache: false })];
                    case 2:
                        files2 = _a.sent();
                        chai_1.expect(files1).to.not.equal(files2);
                        return [2];
                }
            });
        }); });
        it('override file', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files, readmes;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create()
                            .add({ dir: './example/tmpl-2' })
                            .add({ dir: './example/sub-folder/tmpl-3' });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        readmes = files.filter(function (f) { return f.source.endsWith('/README.md'); });
                        chai_1.expect(readmes.length).to.eql(1);
                        chai_1.expect(readmes[0].base.endsWith('/tmpl-3')).to.eql(true);
                        return [2];
                }
            });
        }); });
        it('prepends with [targetDir] path', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files, sources, targets;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create().add({
                            dir: './example/tmpl-2',
                            targetDir: '///foo//bar////',
                        });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        sources = files.map(function (f) { return f.source; });
                        targets = files.map(function (f) { return f.target; });
                        chai_1.expect(sources).to.include('/README.md');
                        chai_1.expect(sources).to.include('/blueprint.png');
                        chai_1.expect(sources).to.include('/index.js');
                        chai_1.expect(targets).to.include('/foo/bar/README.md');
                        chai_1.expect(targets).to.include('/foo/bar/blueprint.png');
                        chai_1.expect(targets).to.include('/foo/bar/index.js');
                        return [2];
                }
            });
        }); });
        it('prepends with [targetDir] path (single file)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl, files, sources, targets;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl = _1.Template.create().add({
                            dir: './example/tmpl-2',
                            pattern: 'README.md',
                            targetDir: '///foo//bar////',
                        });
                        return [4, tmpl.files()];
                    case 1:
                        files = _a.sent();
                        sources = files.map(function (f) { return f.source; });
                        targets = files.map(function (f) { return f.target; });
                        chai_1.expect(sources).to.include('/README.md');
                        chai_1.expect(targets).to.include('/foo/bar/README.md');
                        return [2];
                }
            });
        }); });
    });
    describe('filter', function () {
        it('adds as a new instance', function () {
            var tmpl1 = _1.Template.create({ dir: './tmpl-1' });
            var tmpl2 = tmpl1.filter(function (file) { return true; });
            chai_1.expect(tmpl1).to.not.equal(tmpl2);
        });
        it('applies filter', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var tmpl1, tmpl2, files1, files2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tmpl1 = _1.Template.create({ dir: './example/tmpl-2' });
                        tmpl2 = tmpl1.filter(function (f) { return f.source.endsWith('.js'); });
                        return [4, tmpl1.files()];
                    case 1:
                        files1 = _a.sent();
                        return [4, tmpl2.files()];
                    case 2:
                        files2 = _a.sent();
                        chai_1.expect(files1.length).to.eql(3);
                        chai_1.expect(files2.length).to.eql(1);
                        return [2];
                }
            });
        }); });
    });
    describe('middleware ("use")', function () {
        it('adds as a new instance', function () {
            var tmpl1 = _1.Template.create({ dir: './tmpl-1' });
            var tmpl2 = tmpl1.use(function (req, res) { return true; });
            chai_1.expect(tmpl1).to.not.equal(tmpl2);
        });
        it('process: change => write', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var dir, tmpl, file, _a, isBlueprintBinary;
            var _this = this;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        dir = common_1.fsPath.resolve(TEST_DIR);
                        tmpl = _1.Template.create()
                            .add({ dir: './example/tmpl-2' })
                            .use(function (req, res) {
                            if (!req.isBinary) {
                                chai_1.expect(typeof req.text).to.eql('string');
                            }
                            if (req.isBinary) {
                                chai_1.expect(req.text).to.eql(undefined);
                            }
                            res.replaceText(/__GREETING__/g, req.variables.greeting);
                            res.next();
                        })
                            .use(function (req, res) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var path;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        path = common_1.fsPath.join(dir, req.path.target);
                                        return [4, common_1.fs.ensureDir(dir)];
                                    case 1:
                                        _a.sent();
                                        return [4, common_1.fs.writeFile(path, req.buffer)];
                                    case 2:
                                        _a.sent();
                                        res.complete();
                                        return [2];
                                }
                            });
                        }); });
                        return [4, tmpl.execute({ variables: { greeting: 'Hello!' } })];
                    case 1:
                        _b.sent();
                        _a = {};
                        return [4, common_1.fs.readFile(common_1.fsPath.join(dir, 'index.js'), 'utf8')];
                    case 2:
                        _a.indexJs = _b.sent();
                        return [4, common_1.fs.readFile(common_1.fsPath.join(dir, 'README.md'), 'utf8')];
                    case 3:
                        file = (_a.readme = _b.sent(),
                            _a);
                        return [4, common_1.isBinaryFile(common_1.fsPath.join(dir, 'blueprint.png'))];
                    case 4:
                        isBlueprintBinary = _b.sent();
                        chai_1.expect(file.indexJs).to.include("console.log('Hello!');");
                        chai_1.expect(file.readme).to.include("# tmpl-2");
                        chai_1.expect(isBlueprintBinary).to.eql(true);
                        return [2];
                }
            });
        }); });
        it('process: with path filter', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var paths, tmpl;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        paths = [];
                        tmpl = _1.Template.create()
                            .add('./example/tmpl-1')
                            .use(/\.ts$/, function (req, res) {
                            paths = paths.concat([req.path.target]);
                            res.next();
                        });
                        return [4, tmpl.execute()];
                    case 1:
                        _a.sent();
                        chai_1.expect(paths).to.eql(['/index.ts', '/src/index.ts']);
                        return [2];
                }
            });
        }); });
    });
    describe('dispose', function () {
        it('disposes of a template', function () {
            var tmpl = _1.Template.create();
            var count = 0;
            tmpl.disposed$.subscribe(function () { return count++; });
            chai_1.expect(tmpl.isDisposed).to.eql(false);
            tmpl.dispose();
            chai_1.expect(count).to.eql(1);
            chai_1.expect(tmpl.isDisposed).to.eql(true);
        });
    });
    describe('events$ (observable)', function () {
        it('exposes an events observable', function () {
            var tmpl = _1.Template.create();
            chai_1.expect(tmpl.events$).to.be.an.instanceof(rxjs_1.Observable);
        });
        it('fires alert event from middleware', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var events, tmpl, e;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        events = [];
                        tmpl = _1.Template.create()
                            .add('./example/tmpl-1')
                            .use(function (req, res) {
                            res.alert({ message: 'Foo', path: req.path.source });
                            res.done('NEXT');
                        });
                        tmpl.events$.subscribe(function (e) { return (events = events.concat([e])); });
                        return [4, tmpl.execute()];
                    case 1:
                        _a.sent();
                        chai_1.expect(events.length).to.be.greaterThan(0);
                        e = events
                            .filter(function (e) { return e.type === 'ALERT'; })
                            .map(function (e) { return e.payload; })
                            .find(function (e) { return e.path.endsWith('.babelrc'); });
                        chai_1.expect(e && e.message).to.eql('Foo');
                        chai_1.expect(e && e.path).to.eql('/.babelrc');
                        return [2];
                }
            });
        }); });
    });
    it('fires start/complete events around [execute]', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var events, tmpl, files, first, last;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    events = [];
                    tmpl = _1.Template.create()
                        .add('./example/tmpl-1')
                        .use(function (req, res) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            res.alert({ message: 'middleware' });
                            res.next();
                            return [2];
                        });
                    }); });
                    tmpl.events$.subscribe(function (e) { return (events = events.concat([e])); });
                    return [4, tmpl.files()];
                case 1:
                    files = _a.sent();
                    return [4, tmpl.execute()];
                case 2:
                    _a.sent();
                    chai_1.expect(events.length).to.be.greaterThan(2);
                    first = events[0];
                    last = events[events.length - 1];
                    chai_1.expect(first.type).to.eql('EXECUTE/start');
                    chai_1.expect(last.type).to.eql('EXECUTE/complete');
                    chai_1.expect(first.payload.files).to.eql(files);
                    chai_1.expect(last.payload.files).to.eql(files);
                    return [2];
            }
        });
    }); });
});
